#!/usr/bin/env node
"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const process_1 = require("process");
const config_1 = require("./config");
const index_1 = require("./index");
function main(args) {
    if (args.length < 1) {
        process.stderr.write('Usage: react-native-icon-builder <config-file>\n');
        process.exit(1);
        return;
    }
    process.stdout.write('Building icons ...');
    let config;
    try {
        config = readConfig(args[0]);
    }
    catch (err) {
        process.stdout.write('\n');
        process.stderr.write(`Failed: ${err}\n`);
        process.exit(1);
        return;
    }
    index_1.generate(config)
        .then(generatedFiles => {
        process.stdout.write(' done!\n');
        if (generatedFiles.length === 0) {
            process.stdout.write('No files written.\n');
        }
        else {
            process.stdout.write('Wrote files:\n');
            generatedFiles.forEach(generatedFile => process.stdout.write(`- ${generatedFile}\n`));
        }
    })
        .catch(err => {
        process.stdout.write('\n');
        process.stderr.write(`Failed: ${err}\n`);
        process.exit(1);
    });
}
function readConfig(configFilePath) {
    const configFile = JSON.parse(fs.readFileSync(configFilePath, { encoding: 'utf-8' }));
    return config_1.decodeConfig(configFile);
}
if (require.main === module) {
    main(process_1.argv.slice(2));
}
