"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const klaw_sync_1 = __importDefault(require("klaw-sync"));
function generate(config) {
    return Promise.all([
        generateAndroidResources(config.android),
        generateIosIconset(config.ios),
        generateReactNativeImages(config.assets),
    ]).then(flattenArrays);
}
exports.generate = generate;
const androidIconBaseSize = 48;
const androidDensities = [
    { name: 'ldpi', scale: 0.75 },
    { name: 'mdpi', scale: 1 },
    { name: 'hdpi', scale: 1.5 },
    { name: 'xhdpi', scale: 2 },
    { name: 'xxhdpi', scale: 3 },
    { name: 'xxxhdpi', scale: 4 },
];
const iosIcons = [
    { idiom: 'iphone', scale: 2, size: 20 },
    { idiom: 'iphone', scale: 3, size: 20 },
    { idiom: 'iphone', scale: 2, size: 29 },
    { idiom: 'iphone', scale: 3, size: 29 },
    { idiom: 'iphone', scale: 2, size: 40 },
    { idiom: 'iphone', scale: 3, size: 40 },
    { idiom: 'iphone', scale: 2, size: 60 },
    { idiom: 'iphone', scale: 3, size: 60 },
    { idiom: 'ipad', scale: 1, size: 20 },
    { idiom: 'ipad', scale: 2, size: 20 },
    { idiom: 'ipad', scale: 1, size: 29 },
    { idiom: 'ipad', scale: 2, size: 29 },
    { idiom: 'ipad', scale: 1, size: 40 },
    { idiom: 'ipad', scale: 2, size: 40 },
    { idiom: 'ipad', scale: 1, size: 76 },
    { idiom: 'ipad', scale: 2, size: 76 },
    { idiom: 'ipad', scale: 2, size: 83.5 },
    { idiom: 'ios-marketing', scale: 1, size: 1024, flattenAlpha: true },
];
const reactNativeImageScales = [
    { scale: 1, suffix: '' },
    { scale: 2, suffix: '@2x' },
    { scale: 3, suffix: '@3x' },
];
function generateAndroidResources(androidConfig) {
    if (!androidConfig) {
        return Promise.resolve([]);
    }
    return genaratePngs(androidConfig.icon, androidDensities.map(density => ({
        filePath: getAndroidIconPath(androidConfig.outputDir, density.name),
        size: {
            height: androidIconBaseSize * density.scale,
            type: Size.Absolute,
            width: androidIconBaseSize * density.scale,
        },
    })));
}
function getAndroidIconPath(baseDir, density) {
    return path.join(baseDir, `mipmap-${density}`, 'ic_launcher.png');
}
function generateIosIconset(iosConfig) {
    if (!iosConfig) {
        return Promise.resolve([]);
    }
    const iconsetDir = path.join(iosConfig.outputDir, 'AppIcon.appiconset');
    return Promise.all([
        generateIosIconsetImages(iosConfig.icon, iconsetDir),
        generateIosIconsetManifest(iconsetDir),
    ]).then(flattenArrays);
}
function generateIosIconsetImages(sourceIcon, iconsetDir) {
    return genaratePngs(sourceIcon, iosIcons.map(icon => ({
        filePath: path.join(iconsetDir, getIosIconFilename(icon)),
        flattenAlpha: icon.flattenAlpha,
        size: {
            height: icon.size * icon.scale,
            type: Size.Absolute,
            width: icon.size * icon.scale,
        },
    })));
}
function generateIosIconsetManifest(iconsetDir) {
    const fileName = path.join(iconsetDir, 'Contents.json');
    return fs_extra_1.default
        .ensureDir(iconsetDir)
        .then(() => fs_extra_1.default.writeFile(fileName, JSON.stringify({
        images: iosIcons.map(icon => ({
            filename: getIosIconFilename(icon),
            idiom: icon.idiom,
            scale: `${icon.scale}x`,
            size: `${icon.size}x${icon.size}`,
        })),
        info: {
            author: 'react-native-icon-builder',
            version: 1,
        },
    }, null, 2)))
        .then(() => [fileName]);
}
function getIosIconFilename(icon) {
    return `${icon.idiom}-${icon.size}@${icon.scale}x.png`;
}
function generateReactNativeImages(imagesConfig) {
    if (!imagesConfig) {
        return Promise.resolve([]);
    }
    const outputDir = imagesConfig.outputDir || imagesConfig.inputDir;
    return Promise.all(klaw_sync_1.default(imagesConfig.inputDir, { nodir: true })
        .map(item => item.path)
        .filter(file => path.extname(file) === '.svg')
        .map(file => generateReactNativeImage(file, path.join(outputDir, path.relative(imagesConfig.inputDir, path.dirname(file)))))).then(flattenArrays);
}
function generateReactNativeImage(file, outputDir) {
    return genaratePngs(file, reactNativeImageScales.map(imageScale => ({
        filePath: path.join(outputDir, `${path.basename(file, path.extname(file))}${imageScale.suffix}.png`),
        size: {
            scale: imageScale.scale,
            type: Size.Relative,
        },
    })));
}
var Size;
(function (Size) {
    Size[Size["Absolute"] = 0] = "Absolute";
    Size[Size["Relative"] = 1] = "Relative";
})(Size || (Size = {}));
function genaratePngs(input, outputs) {
    return filterOutputs(input, outputs).then(filteredOutputs => filteredOutputs.length === 0
        ? // Nothing to generate
            Promise.resolve([])
        : // Something to generate
            // Load sharp and input image
            Promise.all([
                Promise.resolve().then(() => __importStar(require('sharp'))).then(sharpImport => sharpImport.default)
                    .then(warmupSharp),
                fs_extra_1.default.readFile(input, { encoding: 'utf-8' }),
            ]).then(([sharp, inputData]) => sharp(Buffer.from(inputData))
                .metadata()
                .then(metadata => Promise.all(filteredOutputs.map(output => genaratePng(sharp, inputData, metadata, output))))));
}
// First run might cause a xmllib error, run safe warmup
// See https://github.com/lovell/sharp/issues/1593
function warmupSharp(sharp) {
    return sharp(Buffer.from(`<svg xmlns="http://www.w3.org/2000/svg"><rect width="1" height="1" /></svg>`, 'utf-8'))
        .metadata()
        .then(() => sharp, () => sharp);
}
function filterOutputs(input, outputs) {
    return Promise.all([
        fs_extra_1.default.stat(input),
        Promise.all(outputs.map(output => fs_extra_1.default
            .stat(output.filePath)
            .catch(() => null)
            .then(outputStat => ({ config: output, stats: outputStat })))),
    ]).then(([inputStat, outputStats]) => outputStats
        .filter(outputStat => outputStat.stats === null ||
        inputStat.mtimeMs > outputStat.stats.mtimeMs)
        .map(outputStat => outputStat.config));
}
function genaratePng(sharp, inputSvg, metadata, output) {
    return fs_extra_1.default.ensureDir(path.dirname(output.filePath)).then(() => {
        if (metadata.format !== 'svg') {
            return Promise.reject(`Unsupported image format ${metadata.format}. Only SVG images are supported.`);
        }
        if (!metadata.density || !metadata.width || !metadata.height) {
            return Promise.reject('Unsupported image, missing size and density');
        }
        let targetDensity;
        switch (output.size.type) {
            case Size.Absolute:
                const inputAspectRatio = metadata.width / metadata.height;
                const outputAspectRatio = output.size.width / output.size.height;
                if (inputAspectRatio !== outputAspectRatio) {
                    return Promise.reject(`Incompatible image aspect ratio. Expected 1:${outputAspectRatio}, got 1:${inputAspectRatio}`);
                }
                targetDensity = (output.size.width / metadata.width) * metadata.density;
                break;
            case Size.Relative:
                targetDensity = output.size.scale * metadata.density;
                break;
            default:
                return assertNever(output.size);
        }
        let image = sharp(Buffer.from(inputSvg), { density: targetDensity });
        if (output.flattenAlpha) {
            image = image.flatten();
        }
        return image
            .png({
            adaptiveFiltering: false,
            compressionLevel: 9,
        })
            .toFile(output.filePath)
            .then(() => output.filePath);
    });
}
function assertNever(x) {
    throw new Error('Unexpected object: ' + x);
}
function flattenArrays(arrays) {
    return [].concat.apply([], arrays);
}
